# coding: utf-8

from objectpack.ui import BaseWindow
from objectpack.ui import BaseListWindow
from objectpack.ui import ObjectTab
from objectpack.ui import TabbedEditWindow

from m3_ext.ui.fields.simple import ExtDateTimeField
from m3_ext.ui import all_components as ext

from sinops import models


class SignAskWin(BaseWindow):

    u"""Окно запроса на подписание действия."""

    def _init_components(self):

        super(SignAskWin, self)._init_components()

        self.display_content = ext.ExtTextArea(
            read_only=True,
        )
        self.center_region = ext.ExtContainer(
            region='center', layout='fit')

        # Данная панель будет содержать грид с сертификатами пользователя
        self.south_region = ext.ExtContainer(
            region='south', height=120, layout='fit')

        self.sign_btn = ext.ExtButton(text=u'Подписать')
        self.cancel_btn = ext.ExtButton(text=u'Отмена')

    def _do_layout(self):
        super(SignAskWin, self)._do_layout()
        self.layout = 'border'
        self.items[:] = [self.center_region, self.south_region]
        self.center_region.items[:] = [self.display_content]
        self.buttons.extend([
            self.sign_btn,
            self.cancel_btn,
        ])
        self.set_params({})

    def set_params(self, params):
        super(SignAskWin, self).set_params(params)
        self.template_globals = 'm3_sinops/sign-ask-window.js'
        self.title = u'Действие необходимо подписать'
        self.width, self.height = 500, 400
        self.modal = True
        self.maximizable = self.minimizable = False


class SOListWin(BaseListWindow):

    u"""Окно списка выполненных операций."""

    def set_params(self, params):
        super(SOListWin, self).set_params(params)
        self.maximized = True


class CertTab(ObjectTab):

    u"""Вкладка с данными сертификата подписи."""

    model = models.Certificate
    field_fabric_params = dict(
        field_list=[
            'content',
            'sha1hash',
            'subject_name',
            'issuer_name',
            'serial_number',
        ],
    )

    def init_components(self, win):
        super(CertTab, self).init_components(win)
        for cnt in self._controls:
            cnt.name = 'certificate.' + cnt.name

        self.validity_begin_field = ExtDateTimeField(
            name='certificate.validity_begin',
            label=u'Не действителен до',
            anchor='100%',
        )

        self.validity_end_field = ExtDateTimeField(
            name='certificate.validity_end',
            label=u'Не действителен после',
            anchor='100%',
        )

    def do_layout(self, win, tab):
        super(CertTab, self).do_layout(win, tab)

        tab.items[:] = [
            self.field__serial_number,
            self.field__subject_name,
            self.field__issuer_name,
            self.validity_begin_field,
            self.validity_end_field,
            self.field__content,
            self.field__sha1hash,
        ]


class SOMainTab(ObjectTab):

    u"""Вкладка с основной информацией о выполненной операции."""

    model = models.Operation

    title = u'Общие сведения'

    field_fabric_params = dict(
        field_list=[
            'signed_message',
            'verbose_content',
            'request_path',
        ],
    )

    def init_components(self, win):
        super(SOMainTab, self).init_components(win)

        self.otype_code = ext.ExtStringField(
            name='operation_type.code',
            label=u'Код операции',
            anchor='100%',
        )

        self.otype_description = ext.ExtTextArea(
            name='operation_type.description',
            label=u'Описание операции',
            anchor='100%',
            height=40,
        )
        self.signing_time_field = ExtDateTimeField(
            name='signing_time',
            label=u'Время подписания',
            anchor='100%',
        )
        self.created_time_field = ExtDateTimeField(
            name='created',
            label=u'Время выполнения',
            anchor='100%',
        )

    def do_layout(self, win, tab):
        super(SOMainTab, self).do_layout(win, tab)

        tab.items[:] = [
            self.otype_code,
            self.otype_description,
            self.field__verbose_content,
            self.field__request_path,
            self.signing_time_field,
            self.created_time_field,
            self.field__signed_message,
        ]
        tab.label_width = 120

    def set_params(self, win, params):
        super(SOMainTab, self).set_params(win, params)
        self.field__signed_message.style = {'font-size': '0.9em'}


class SOEditWin(TabbedEditWindow):

    u"""Окно просмотра информации о выполненной операции."""

    tabs = [
        SOMainTab,
        CertTab,
    ]

    def set_params(self, params):
        super(SOEditWin, self).set_params(params)
        self.width = 800
        self.make_read_only()
